==> 1.4.8.tar.gz: lib/src/state_observer.dart <==
part of '../flutterx_live_data.dart';

/// Link [LiveData] observers to the lifecycle of your [State]
///
/// Implement this mixin on your widget's [State] to add [Observer]s to a [LiveData] within the state lifecycle
/// Every LiveData can be observed inside [registerObservers] method that you can override
/// The observer must be added through proper extension methods and will be removed automatically on state [dispose]
/// For better performances do data.observe(_myCallbackFunction) instead of data.observe((value) => ...)
/// Check [StateObserverExt]
mixin StateObserver<T extends StatefulWidget> on State<T> {
  final Map<_ObserverKey, Disposable> _disposables = {};
  Set<_ObserverKey>? _keys;

  @override
  void initState() {
    super.initState();
    doRegister();
  }

  @override
  void didUpdateWidget(T oldWidget) {
    super.didUpdateWidget(oldWidget);
    doRegister();
  }

  @override
  void dispose() {
    doUnregister();
    super.dispose();
  }

  /// Override this method to observe you LiveData(s)
  @protected
  FutureOr<void> registerObservers();

  /// Trigger calling [registerObservers] and automatically manage registering/disposing observers
  @protected
  void doRegister() {
    if (_keys != null) return;
    final keys = _keys = _disposables.keys.toSet();
    final result = registerObservers();
    void doComplete() {
      for (final key in keys) _disposables.remove(key)!.dispose();
      _keys = null;
    }

    result is Future ? result.whenComplete(doComplete) : doComplete();
  }

  /// Dispose all active observers
  @protected
  void doUnregister() {
    for (final key in _disposables.keys.toList(growable: false)) _disposables.remove(key)!.dispose();
  }

  void _observeAll(_CombinedObserverKey key, Iterable<LiveData> sources, void Function(List values) onChanged) {
    final keys = _keys;
    if (keys == null) throw StateError('observe can only be called within registerObservers function');
    _disposables.containsKey(key) ? keys.remove(key) : _disposables[key] = observeAllForever(sources, onChanged);
  }

  void _observe<V>(_ObserverKey key, LiveData<V> data, Observer<V> onChanged) {
    final keys = _keys;
    if (keys == null) throw StateError('observe can only be called within registerObservers function');
    _disposables.containsKey(key) ? keys.remove(key) : _disposables[key] = data.observeForever(onChanged);
  }
}

@immutable
class _ObserverKey {
  final Object _data;
  final Object _onChanged;

  const _ObserverKey(this._data, this._onChanged);

  @override
  bool operator ==(Object other) =>
      other is _ObserverKey && identical(other._data, _data) && other._onChanged == _onChanged;

  @override
  int get hashCode {
    var hash = 0;
    hash = 0x1fffffff & identityHashCode(_data);
    hash = 0x1fffffff & (hash + ((0x0007ffff & hash) << 10));
    hash = hash ^ (hash >> 6);
    hash = 0x1fffffff & (hash + _onChanged.hashCode);
    hash = 0x1fffffff & (hash + ((0x0007ffff & hash) << 10));
    hash = hash ^ (hash >> 6);
    hash = 0x1fffffff & (hash + ((0x03ffffff & hash) << 3));
    hash = hash ^ (hash >> 11);
    return 0x1fffffff & (hash + ((0x00003fff & hash) << 15));
  }
}

@immutable
class _CombinedObserverKey extends _ObserverKey {
  const _CombinedObserverKey(List<LiveData> sources, Object onChanged) : super(sources, onChanged);

  @override
  bool operator ==(Object other) {
    if (other is! _CombinedObserverKey || other._onChanged != _onChanged) return false;
    final a = _data as List;
    final b = other._data as List;
    if (a.length != b.length) return false;
    if (identical(a, b)) return true;
    for (var index = 0; index < a.length; index++) if (!identical(a[index], b[index])) return false;
    return true;
  }

  @override
  int get hashCode {
    var hash = 0;
    for (final source in _data as List) {
      hash = 0x1fffffff & (hash + identityHashCode(source));
      hash = 0x1fffffff & (hash + ((0x0007ffff & hash) << 10));
      hash = hash ^ (hash >> 6);
    }
    hash = 0x1fffffff & (hash + _onChanged.hashCode);
    hash = 0x1fffffff & (hash + ((0x0007ffff & hash) << 10));
    hash = hash ^ (hash >> 6);
    hash = 0x1fffffff & (hash + ((0x03ffffff & hash) << 3));
    hash = hash ^ (hash >> 11);
    return 0x1fffffff & (hash + ((0x00003fff & hash) << 15));
  }
}

==> 1.5.4.tar.gz: lib/src/state_observer.dart <==
part of '../flutterx_live_data.dart';

/// Link [LiveData] observers to the lifecycle of your [State]
///
/// Implement this mixin on your widget's [State] to add [Observer]s to a [LiveData] within the state lifecycle
/// Every LiveData can be observed inside [registerObservers] method that you can override
/// The observer must be added through proper extension methods and will be removed automatically on state [dispose]
/// For better performances do data.observe(_myCallbackFunction) instead of data.observe((value) => ...)
/// Check [StateObserverExt]
mixin StateObserver<T extends StatefulWidget> on State<T> {
  final Map<_ObserverKey, Disposable> _disposables = {};
  Set<_ObserverKey>? _keys;

  @override
  void initState() {
    super.initState();
    doRegister();
  }

  @override
  void didUpdateWidget(T oldWidget) {
    super.didUpdateWidget(oldWidget);
    doRegister();
  }

  @override
  void dispose() {
    doUnregister();
    super.dispose();
  }

  /// Override this method to observe you LiveData(s)
  @protected
  FutureOr<void> registerObservers();

  /// Trigger calling [registerObservers] and automatically manage registering/disposing observers
  @protected
  void doRegister() {
    if (_keys != null) return;
    final keys = _keys = _disposables.keys.toSet();
    final result = registerObservers();
    void doComplete() {
      for (final key in keys) _disposables.remove(key)!.dispose();
      _keys = null;
    }

    result is Future ? result.whenComplete(doComplete) : doComplete();
  }

  /// Dispose all active observers
  @protected
  void doUnregister() {
    for (final key in _disposables.keys.toList(growable: false)) _disposables.remove(key)!.dispose();
  }

  void _observeAll(_CombinedObserverKey key, Iterable<LiveData> sources, void Function(List values) onChanged,
      Dispatcher? dispatcher) {
    final keys = _keys;
    if (keys == null) throw StateError('observe can only be called within registerObservers function');
    _disposables.containsKey(key)
        ? keys.remove(key)
        : _disposables[key] = observeAllForever(sources, onChanged, dispatcher: dispatcher);
  }

  void _observe<V>(_ObserverKey key, LiveData<V> data, Observer<V> onChanged, Dispatcher? dispatcher) {
    final keys = _keys;
    if (keys == null) throw StateError('observe can only be called within registerObservers function');
    _disposables.containsKey(key)
        ? keys.remove(key)
        : _disposables[key] = data.observeForever(onChanged, dispatcher: dispatcher);
  }
}

@immutable
class _ObserverKey {
  final Object _data;
  final Object _onChanged;

  const _ObserverKey(this._data, this._onChanged);

  @override
  bool operator ==(Object other) =>
      other is _ObserverKey && identical(other._data, _data) && other._onChanged == _onChanged;

  @override
  int get hashCode {
    var hash = 0;
    hash = 0x1fffffff & identityHashCode(_data);
    hash = 0x1fffffff & (hash + ((0x0007ffff & hash) << 10));
    hash = hash ^ (hash >> 6);
    hash = 0x1fffffff & (hash + _onChanged.hashCode);
    hash = 0x1fffffff & (hash + ((0x0007ffff & hash) << 10));
    hash = hash ^ (hash >> 6);
    hash = 0x1fffffff & (hash + ((0x03ffffff & hash) << 3));
    hash = hash ^ (hash >> 11);
    return 0x1fffffff & (hash + ((0x00003fff & hash) << 15));
  }
}

@immutable
class _CombinedObserverKey extends _ObserverKey {
  const _CombinedObserverKey(List<LiveData> sources, Object onChanged) : super(sources, onChanged);

  @override
  bool operator ==(Object other) {
    if (other is! _CombinedObserverKey || other._onChanged != _onChanged) return false;
    final a = _data as List;
    final b = other._data as List;
    if (a.length != b.length) return false;
    if (identical(a, b)) return true;
    for (var index = 0; index < a.length; index++) if (!identical(a[index], b[index])) return false;
    return true;
  }

  @override
  int get hashCode {
    var hash = 0;
    for (final source in _data as List) {
      hash = 0x1fffffff & (hash + identityHashCode(source));
      hash = 0x1fffffff & (hash + ((0x0007ffff & hash) << 10));
      hash = hash ^ (hash >> 6);
    }
    hash = 0x1fffffff & (hash + _onChanged.hashCode);
    hash = 0x1fffffff & (hash + ((0x0007ffff & hash) << 10));
    hash = hash ^ (hash >> 6);
    hash = 0x1fffffff & (hash + ((0x03ffffff & hash) << 3));
    hash = hash ^ (hash >> 11);
    return 0x1fffffff & (hash + ((0x00003fff & hash) << 15));
  }
}
